// CAROUSTORE 1.0 by MagicManu
// 11/2016
// Video : https://youtu.be/_vXFOUmiD2g
// http://blog.magicmanu.com

// VNH2SP30 ​Monster Motor Shield (wiper motor)
int InEN = 2;        // Enable
int InCS = 3;        // Current sensor
int InNA = 4;        // Clockwise
int InNB = 5;        // Counterclockwise
int InPW = 6;        // Power (output)
int PWM_val = 0;     // Power value
int MaxSpeed = 255;  // Max speed of the motor

// 74HC165 shift registers (8 buttons)
int LATCH = 10; // pin 1 (Load)
int CLOCK = 9;  // pin 2 (Clock)
int DATA = 8;   // pin 9 (Q7)
int Buttons[9]; // Buttons[1] to Buttons[8] (LOW or HIGH)

// Position sensor
int InSensorInit = 11;   // Input of Init sensor (pos #1)
int InSensorPos  = 12;   // Input of Position sensor (8 for 1 lap)
int LastSensorPos = 0;   // Last State of position sensor
int ActualPosition = 0;  // Actual Position (0 = uninitialized)
long TimeRefSensor = 0;  // Time at the sensor is on
int CptSensor = 1000;    // Number of millisecond of the sensor is on

boolean Rotation = LOW;  // Motor rotate ?
int Direction = 0;       // 1=CW  2=CCW
int Request = 0;         // Request position
long TimeRef = 0;        // Time at the motor start
int Cpt = 0;             // Number of millisecond of the motor start or the light on

// LED
int OutLED = 13;         // Output of the LED
boolean LightOn = LOW;   // Turn on the LED ?


void setup() {
  // VNH2SP30
  pinMode(InEN, OUTPUT);
  pinMode(InNA, OUTPUT);
  pinMode(InNB, OUTPUT);
  pinMode(InPW, OUTPUT);
  
  // 74HC165
//Serial.begin(57600);
  pinMode(LATCH, OUTPUT);
  pinMode(CLOCK, OUTPUT);
  pinMode(DATA, INPUT);
  digitalWrite(CLOCK, LOW);
  digitalWrite(LATCH, LOW);
  
  // Position sensor  
  pinMode(InSensorInit, INPUT_PULLUP);
  pinMode(InSensorPos, INPUT_PULLUP);
  
  // LED
  pinMode(OutLED, OUTPUT);  
}


void loop() { 
    
  // Read sensors
  int SensorInit = !digitalRead(InSensorInit);  
  int SensorPos = !digitalRead(InSensorPos); 
  
  CptSensor = millis() - TimeRefSensor;  // Number of millisecond of the last sensor detected (avoid double detection)

  // Update position (on rising edge of the sensor)
  if (Direction != 0 && LastSensorPos != SensorPos && CptSensor > 300){
    if (SensorPos){

      if (Direction == 1 && ActualPosition > 0){
        if (ActualPosition >= 8)
          ActualPosition = 1;
        else
          ActualPosition++;   // Increment
       }
      else if (Direction == 2){
        if (ActualPosition <=1)
          ActualPosition = 8;
        else
          ActualPosition--;   // Decrement
      }
      
      if (SensorInit)
        ActualPosition = 1  ;  // Init pos
        
        
      TimeRefSensor = millis();  
    }
    LastSensorPos = SensorPos;
  }


  // For the first time
  if (ActualPosition == 0 && SensorInit)
    ActualPosition = 1  ;  // Init pos
  
    
  // Read buttons
  digitalWrite(LATCH, HIGH);
  for (int i = 0; i <= 7; i++){
    Buttons[8-i] = digitalRead(DATA);
    digitalWrite(CLOCK, HIGH);
    digitalWrite(CLOCK, LOW);   
  } 
  digitalWrite(LATCH, LOW);
  
  
  // Button pressed ? (Other than current and not already in rotation)
  for (int i = 1; i <= 8; i++){
    if (Buttons[i] && !Rotation){
      if (i != ActualPosition){
        Request = i;
        TimeRef = millis();
        TimeRefSensor = millis();
        
        // In the nearest direction
        Direction = 1; // CW
        if (ActualPosition != 0){
          for (int p = 1; p < 4; p++){
              // Counterclockwise for 3 positions under the current
              int OkForCCW = ActualPosition - p < 1 ? ActualPosition - p + 8 : ActualPosition - p;
              if (Request == OkForCCW){
                  Direction = 2;   // CCW
                  break;
              }            
          } 
        }
      }
      else{
        // Just turn on the light if already on position
        TimeRef = millis();
        LightOn = HIGH;        
      }
    }
    
  }
  
  // Stop the motor
  if ((Rotation && ActualPosition == Request) || Cpt > 15000){
    PWM_val = 0;
    digitalWrite(InEN, LOW);
    digitalWrite(InNA, LOW);
    digitalWrite(InNB, LOW);  
    analogWrite(InPW, PWM_val);    
    Rotation = LOW;  
    Direction = 0;
    TimeRef = millis();    // For light
    
    if (Cpt > 15000){
      ActualPosition = 0;  // Error, sensor not find or time too long
      Cpt = 0;
    }
    else
      LightOn = HIGH;
  }
  
  
  // Turn on the LED for 10 seconds
  if (LightOn){
    digitalWrite(OutLED, HIGH);    // LED on
    Cpt = millis() - TimeRef;
    
    if (Cpt > 10000){
      digitalWrite(OutLED, LOW);   // LED off
      LightOn = LOW;
    }
  }  
  
  // Start the motor
  if (Direction > 0){
    LightOn = LOW;
    digitalWrite(OutLED, LOW);   // LED off
    
    if (Direction == 1){
      digitalWrite(InNA, HIGH);  // CW
      digitalWrite(InNB, LOW);        
    }
    else{           
      digitalWrite(InNA, LOW);
      digitalWrite(InNB, HIGH);  // CCW        
    }   
    digitalWrite(InEN, HIGH);
    
    MaxSpeed = (abs(ActualPosition - Request) == 1 || abs(ActualPosition - Request) == 7) ? 150 : 255; // Slow-down
    analogWrite(InPW, PWM_val);

    // Acceleration
    Cpt = millis() - TimeRef;    
    PWM_val = Cpt / 5;
    if (PWM_val > MaxSpeed)
      PWM_val = MaxSpeed;
    
    Rotation = HIGH;
  }  
}
 
